#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <assert.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <sys/un.h>
#include <stdarg.h>

#include "../../sr_module.h"
#include "../../mem/mem.h"
#include "../../dprint.h"
#include "../../str.h"
#include "../../pvar.h"
#include "../../ut.h"
#include "../../script_cb.h"
#include "../../parser/digest/digest.h"
#include "../../parser/parse_from.h"
#include "../dialog/dlg_load.h"
#include "../dialog/dlg_hash.h"
#include "../../mod_fix.h"

/* Function prototypes */
static int mod_init(void);
static int child_init(int rank);
static void destroy(void);

static int test6(struct sip_msg *msg, char *arg1, char *arg2, char* arg3, char* arg4, char* arg5, char* arg6);
static int pvn_fixup_spve_spve_6(void** param, int param_no);

static cmd_export_t commands[] = {
    {"test6",  (cmd_function)test6, 6, pvn_fixup_spve_spve_6, 0, 
               REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE |
               ERROR_ROUTE | LOCAL_ROUTE | STARTUP_ROUTE | TIMER_ROUTE  },
    {0, 0, 0, 0, 0, 0}
};

struct module_exports exports = {
    "test",  // module name
    MODULE_VERSION,  // module version
    DEFAULT_DLFLAGS, // dlopen flags
    commands,        // exported functions
    NULL,      // exported parameters
    NULL,            // exported statistics
    NULL,            // exported MI functions
    NULL,            // exported pseudo-variables
    NULL,            // extra processes
    mod_init,        // module init function (before fork. kids will inherit)
    NULL,            // reply processing function
    destroy,         // destroy function
    child_init       // child init function
};

static int
mod_init(void)
{
	return 0;
}

static int
child_init(int rank)
{
    return 0;
}

static void
destroy(void) 
{
}

/*! \brief
 * fixup for functions that get 6 parameters
 * - the parameters are converted to gparam_t (str or pv_elem_t)
 */
static int pvn_fixup_spve_spve_6(void** param, int param_no)
{
	if (!(param_no >= 1 && param_no <= 6))
	{
		LM_ERR("invalid parameter number %d\n", param_no);
		return E_UNSPEC;
	}
	
	return fixup_spve(param);
}

#define DEFINE_STRING_PARAM(param) str param##_s = {NULL, 0}

#define GET_VALUE_OF_STRING_PARAM(param) \
{ \
	if(param == NULL || fixup_get_svalue(msg, (gparam_p)param, &param##_s) != 0) \
	{ \
		LM_ERR("Invalid "#param" argument\n"); \
		return -1; \
	} \
}

#define SHOW_VALUE_OF_STRING_PARAM(param) LM_INFO(#param": [%.*s]", param##_s.len, param##_s.s)

static int 
test6(struct sip_msg *msg, char *arg1, char *arg2, char* arg3, char* arg4, char* arg5, char* arg6)
{
	DEFINE_STRING_PARAM(arg1);
	DEFINE_STRING_PARAM(arg2);
	DEFINE_STRING_PARAM(arg3);
	DEFINE_STRING_PARAM(arg4);
	DEFINE_STRING_PARAM(arg5);
	DEFINE_STRING_PARAM(arg6);
	
	GET_VALUE_OF_STRING_PARAM(arg1);
    GET_VALUE_OF_STRING_PARAM(arg2);
    GET_VALUE_OF_STRING_PARAM(arg3);
    GET_VALUE_OF_STRING_PARAM(arg4);
    GET_VALUE_OF_STRING_PARAM(arg5);
    GET_VALUE_OF_STRING_PARAM(arg6);

	SHOW_VALUE_OF_STRING_PARAM(arg1);
    SHOW_VALUE_OF_STRING_PARAM(arg2);
    SHOW_VALUE_OF_STRING_PARAM(arg3);
    SHOW_VALUE_OF_STRING_PARAM(arg4);
    SHOW_VALUE_OF_STRING_PARAM(arg5);
    SHOW_VALUE_OF_STRING_PARAM(arg6);
    
    return 0;
}
